/*
    Console.h, Copyright (c) by Silvio Dragone & David Reist,
    Swiss Federal Institute of Technology,
    Computer Engineering and Networks Laboratory.
 
    TOPSY -- A Teachable Operating System.
             Implementation of a tiny and simple
             micro kernel for teaching purposes.
 
    For further information, please visit http://www.tik.ee.ethz.ch/~topsy
 
    This software is provided under the terms of the GNU General Public Licence.
 
    A full copy of the GNU GPL is provided in the file COPYING found in the 
    development root of Topsy.
*/

#ifndef _IXP_MEM_ACCESS_H_
#define _IXP_MEM_ACCESS_H_


#include "types.h"


void MemAccess_Zero(address_t regionAddr, uint32 startAddr, int bytes);
void MemAccess_Init(void);
inline uint32 memRead32(uint32 address);
inline void memWrite32(uint32 address, uint32 value);
inline uint8 memRead8(uint32 address);
inline void memWrite8(uint32 address, uint8 value);
inline uint32 dramRead32(uint32 address);
inline uint32 sramRead32(uint32 address);
inline uint32 scratchRead32(uint32 address);
inline void dramWrite32(uint32 address, uint32 value);
inline void sramWrite32(uint32 address, uint32 value);
inline void scratchWrite32(uint32 address, uint32 value);


#endif

